#include "Arduino.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"
#define MAXVOLUME 30
#define MAX_INITIALISATION_ATTEMPTS 5
//#define volumeControl

SoftwareSerial softwareSerial1(10, 11); // RX, TX
SoftwareSerial softwareSerial2(2, 3); // RX, TX
DFRobotDFPlayerMini Player1;
DFRobotDFPlayerMini Player2;

struct player {
  String name;
  int button1pin;
  int button2pin;
  int potPin;
  bool button1State;
  bool button2State;
  int volume;
  SoftwareSerial* serial;
  DFRobotDFPlayerMini &player;
};

player player1 = {"player1", 4, 5, A0, HIGH, HIGH, 0, &softwareSerial1, Player1};
player player2 = {"player2", 6, 7, A1, HIGH, HIGH, 0, &softwareSerial2, Player2};


void printDetail(uint8_t type, int value);

void setup()
{
  Serial.begin(115200);
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, LOW);
  delay(300);
  initialise(player1);
  delay(2000);
  initialise(player2);
  delay(2000);
  digitalWrite(LED_BUILTIN, HIGH);
}

void loop() {
  DFRobotDFPlayerMini *test = &player1.player;
  playerHandle(&player1);
  playerHandle(&player2);
  delay(100);
}

void initialise(player DFplayer) {
  pinMode(DFplayer.button1pin, INPUT_PULLUP);
  pinMode(DFplayer.button2pin, INPUT_PULLUP);
  for (int i=0; i< MAX_INITIALISATION_ATTEMPTS; i++) {
    DFplayer.serial->begin(9600);
    if (!DFplayer.player.begin(*DFplayer.serial)) {  //Use softwareSerial to communicate with mp3.
      Serial.println("Unable to begin " + DFplayer.name);
      Serial.println(F("1.Please recheck the connection!"));
      Serial.println(F("2.Please insert the SD card!"));
      }
    else {
      
      Serial.println(F("DFPlayer Mini online."));
      DFplayer.player.setTimeOut(500); //Set serial communication time out 500ms
      //----Set volume----
      DFplayer.player.volume(MAXVOLUME);  //Set volume value (0~30).
      //----Set different EQ----
        DFplayer.player.EQ(DFPLAYER_EQ_BASS);
      //----Set device we use SD as default----
      DFplayer.player.outputDevice(DFPLAYER_DEVICE_SD);
      //----Mp3 control----
      DFplayer.player.enableDAC();  //Enable On-chip DAC
      delay(300);
      DFplayer.player.play(1);
      delay(3000);
      DFplayer.player.stop();
      return;
    }
    //delay(2000);
  }
  Serial.println("Unable to initialise "+ DFplayer.name+ ", rebooting\n\n");
  delay(300);
  asm volatile ("  jmp 0"); 
}


void playerHandle (player *DFplayer) {
  bool button1 = digitalRead(DFplayer->button1pin);
  bool button2 = digitalRead(DFplayer->button2pin);
  int potValue = analogRead(DFplayer->potPin);
  DFRobotDFPlayerMini Player = DFplayer->player;
  if (button1 == LOW && DFplayer->button1State == HIGH) Player.play(1);
  if (button2 == LOW && DFplayer->button2State == HIGH) Player.play(2);
  #ifdef volumeControl
  int volume = map(potValue, 0, 1023, 0, MAXVOLUME);
  if (volume != DFplayer->volume) {Player.volume(volume); Serial.println(volume);}
  DFplayer->volume = volume;
  #endif
  DFplayer->button1State = button1;
  DFplayer->button2State = button2;
}
